/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/
#ifndef SSFT_DIR_LIST_H
#define SSFT_DIR_LIST_H

#ifndef FILE_INFO_H
#   include "FileInfo.h"
#endif

class TransferException;

class DirList
{
public:
    typedef std::set<std::string> pathset_t;

    enum DirTagsE
    {
        TAG_DIR_HDR,
        TAG_NODE,
        TAG_END
    };

    DirList();
    virtual ~DirList();

    void Open(const char *path, 
        int flags,
        const char *rootPath = NULL,
        const char *hashDigest = NULL) 
        throw(SsftDisconnect, TransferException);

    // Get a list over files. Used when we do replication.
    // The server should not use this methos, as
    // it may consume lots of memory (and is
    // not optimized for speed).
    void GetList(fileset_t& dstList)
        throw(SsftDisconnect, TransferException);

    // Get the next buffer with data to send. Data is added
    // to the end of the buffer until the end of the 
    // listing, or until the buffer is >= preffered_size
    void GetPart(XmitBuffer& buffer, 
        const size_t preffered_size = (1024 * 8)) 
        throw(SsftDisconnect, TransferException);

    bool AddNodeToBuffer(XmitBuffer& buffer)
      throw(SsftDisconnect, TransferException);

    bool IsEof() const;

    const EVP_MD *m_Md;

private:
    void Reset();
    std::string GetFakePath();
    void GetHash(std::string& hash, const char *name);
    void CloseDir();
    
    int mFlags;
    bool mIsVirgin;
    bool mIsEof;
    std::string mCurrentPath;
    std::string mRootPath; // What the client sees
    std::string mHashDigest;

    pathset_t mVisitedDirs;
    pathset_t mDirsToVisit;

#ifdef WIN32
    HANDLE mHandle; // FindFirstFile() handle
    WIN32_FIND_DATA mData;
#else
    DIR *mDirp;
#endif

};

#endif // SSFT_DIR_LIST_H
