/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/


#ifndef USER_H
#define USER_H


class IdItem
{
public:
    IdItem(int nid, const std::string& value)
    : mId(nid), mValue(value)
    {
    }

    IdItem(const IdItem& from)
    {
        operator = (from);
    }

    void operator = (const IdItem& from)
    {
        mId = from.mId;
        mValue = from.mValue;
    }

    inline const int GetNid() const
    {
        return mId;
    }

    inline const std::string& GetValue() const
    {
        return mValue;
    }

private:
    int mId;
    std::string mValue;
};

class User  
{
public:
    typedef std::list<IdItem> iditemlist_t;
    typedef std::vector<ConfToken> tokenlist_t;

    User();
	User(const char *idString);
	virtual ~User();

    const std::string& GetRootPath() const
    {
        return mRootPath;
    }

    void operator = (const User& v);

    bool operator == (const X509 *psert) const;

    ConfToken *GetTokens()
    {
        return &mTokenList[0];
    }

    const std::string& GetAliasName() const
    {
        return mUserAlias;
    }

    const std::string& GetPerms() const
    {
        return mPerms;
    }

    const std::string& GetOwnerUser() const
    {
        return mOwnerUser;
    }

    const std::string& GetOwnerGroup() const
    {
        return mOwnerGroup;
    }

    const int GetUnixModeFile() const
    {
        return mUnixModeFile;
    }

    const int GetUnixModeDir() const
    {
        return mUnixModeDir;
    }

    bool HaveRight(const char *name);


    std::string DebugInfo() const;


private:
    std::string mUserAlias;
    std::string mRootPath;
    std::string mPerms;
    iditemlist_t mIdPatterns;
    tokenlist_t mTokenList;
    int mUnixModeFile;
    int mUnixModeDir;
    std::string mOwnerUser; // Set ownership on new files
    std::string mOwnerGroup; // Set ownership on new files

    void CreateTokenList();
};

typedef std::list<User *> userptr_list_t;

#endif // USER_H
