/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#include "StdAfx.h"
#include "ssft.h"

using namespace std;


// openssl wrapper for easy certificate management

int do_certs()
{
    string command = "openssl";
    string use_client_key = cli_key_file;
    string use_client_cert = cli_cert_file;
    string new_cert_name = "newreq.pem";

    if (!sign_cert.empty())
    {
        if (!is_file(sign_cert))
        {
            log << LOG_DATE
                << "Certificate name \""
                << sign_cert
                << "\" not found."
                << ENDL;
            return -1;
        }
        cert_new_cli = 2;

        use_client_cert = "";
        const char *p = sign_cert.c_str();
        while(*p && (('.' != *p) || strchr(p+1, '.')))
            use_client_cert += *p++;
        use_client_cert += ".cert";
    }
    
    if (!cert_new_ca && !cert_new_cli && !cert_new_svr)
    {
        log << LOG_DATE 
            << "do_certs(): No options specified. Dunno what to do."
            << ENDL;
        
        return -1;
    }
    
    if (cert_new_ca)
    {
        // Create a new CA
        if (is_file(rootcert_file))
        {
            log << "The root CA already exist. You must delete it manually "
                "before you can create a new one!: \""
                << rootcert_file.c_str()
                << "\""
                << ENDL;
            
            return -1;
        }
        
        // Create a serial file
        if (!is_file(resolve_path("serial")))
        {
            ofstream serial(resolve_path("serial").c_str());
            if (serial)
                serial << "01" << endl;
        }
        
        if (!is_file(resolve_path("index.txt")))
        {
            ofstream serial(resolve_path("index.txt").c_str());
        }
        
        mkdir(resolve_path("certs").c_str() MKDIR_MODE);
        mkdir(resolve_path("crl").c_str() MKDIR_MODE);
        mkdir(resolve_path("newcerts").c_str() MKDIR_MODE);
        mkdir(resolve_path("private").c_str() MKDIR_MODE);
        
        // Create CA
        
        
        if (!is_file(resolve_path("root_ca.conf")))
        {
            ofstream conf_file(resolve_path("root_ca.conf").c_str());
            
            conf_file << "[ req ]" << endl
                << "default_bits = 1024" << endl
                << "default_keyfile = ca.key" << endl
                << "distinguished_name = req_distinguished_name" << endl
                << "x509_extensionst= v3_ca" << endl
                << "string_mask = nombstr" << endl
                << "req_extensions = v3_req" << endl
                << "[ req_distinguished_name ]" << endl
                << "countryName = Country Name (2 letter code)" << endl
                << "countryName_default = NO" << endl
                << "endlcountryName_min = 2" << endl
                << "countryName_max = 2" << endl
                << "stateOrProvinceName = State or Province Name (full name)" << endl
                << "stateOrProvinceName_default = Hordaland" << endl
                << "localityName = Locality Name (eg, city)" << endl
                << "localityName_default = Bergen" << endl
                << "0.organizationName = Organization Name (eg, company)" << endl
                << "0.organizationName_default = Jgaas Internet" << endl
                << "organizationalUnitName = Organizational Unit Name (eg, section)" << endl
                << "organizationalUnitName_default = Certification Services Division" << endl
                << "commonName = Common Name (eg, MD Root CA)" << endl
                << "commonName_max = 64" << endl
                << "commonName_default = Jarle Aase" << endl
                << "emailAddress = Email Address" << endl
                << "emailAddress_max = 40" << endl
                << "emailAddress_default = ca@jgaa.com" << endl
                << "[ v3_ca ]" << endl
                << "basicConstraints = critical,CA:true" << endl
                << "subjectKeyIdentifier = hash" << endl
                << "[ v3_req ]" << endl
                << "nsCertType = objsign,email,server" << endl;
        }
        
        strstream cmd;

        cout << "============================================" << endl
            << "Creating the root certificate." << endl;
        
        cmd << command
            << " req"
            << " -config " << resolve_path("root_ca.conf")
            << " -new "
            << " -x509 "
            << " -keyout " << resolve_path("private") << '/' << "cakey.pem"
            << " -out " << rootcert_file
            << " -days 3650"
            << '\0';
        
        if (verbose_flag)
            clog << "Running command: " << cmd.str() << endl;
        
        if (system(cmd.str()) != 0)
        {
            clog << "Failed to create root certificate using command: "
                << cmd.str();
            
            return -1;
        }
    }

    if (cert_new_svr)
    {
        // Create new server certificate
        if (!is_file(resolve_path("server_cert.conf")))
        {
            ofstream conf_file(resolve_path("server_cert.conf").c_str());
            
            conf_file << "# Generated file" << endl
                << "[ ca ]" << endl
                << "default_ca	= CA_default" << endl
                << "[ CA_default ]" << endl
                << "dir	= " << cert_dir << endl
                << "certs = $dir/certs" << endl
                << "crl_dir = $dir/crl" << endl
                << "database = $dir/index.txt" << endl
                << "new_certs_dir = $dir/newcerts" << endl
                << "certificate	= " << rootcert_file << endl
                << "serial = $dir/serial" << endl
                << "crl	= $dir/crl.pem" << endl
                << "private_key = $dir/private/cakey.pem" << endl
                << "RANDFILE = $dir/private/rand" << endl
                << "default_days = 3650" << endl
                << "default_crl_days = 30" << endl
                << "default_md = md5" << endl
                << "preserve = no" << endl
                << "policy = policy_match" << endl
                << "# For the CA policy" << endl
                << "[ policy_match ]" << endl
                << "countryName = match" << endl
                << "stateOrProvinceName = match" << endl
                << "organizationName = match" << endl
                << "organizationalUnitName = optional" << endl
                << "commonName = supplied" << endl
                << "emailAddress = optional" << endl
                << "# For the 'anything' policy" << endl
                << "# At this point in time, you must list all acceptable 'object'" << endl
                << "# types." << endl
                << "[ policy_anything ]" << endl
                << "countryName = optional" << endl
                << "stateOrProvinceName = optional" << endl
                << "localityName = optional" << endl
                << "organizationName = optional" << endl
                << "organizationalUnitName	= optional" << endl
                << "commonName = supplied" << endl
                << "emailAddress = optional" << endl
                << "[ policy_server ]" << endl
                << "countryName = optional" << endl
                << "stateOrProvinceName = optional" << endl
                << "localityName = optional" << endl
                << "organizationName = supplied" << endl
                << "organizationalUnitName	= optional" << endl
                << "commonName = optional" << endl
                << "emailAddress = optional" << endl
                << "[ req ]" << endl
                << "default_bits = 1024" << endl
                << "default_keyfile = ssft_server.key" << endl
                << "distinguished_name = req_distinguished_name" << endl
                << "string_mask = nombstr" << endl
                << "req_extensions = v3_req" << endl
                << "[ req_distinguished_name ]" << endl
                << "countryName = Country Name (2 letter code)" << endl
                << "countryName_default = NO" << endl
                << "endlcountryName_min = 2" << endl
                << "countryName_max = 2" << endl
                << "stateOrProvinceName = State or Province Name (full name)" << endl
                << "stateOrProvinceName_default = Hordaland" << endl
                << "localityName = Locality Name (eg, city)" << endl
                << "localityName_default = Bergen" << endl
                << "0.organizationName = Organization Name (eg, company)" << endl
                << "0.organizationName_default = Jgaas Internet" << endl
                << "organizationalUnitName = Organizational Unit Name (eg, section)" << endl
                << "organizationalUnitName_default = Certification Services Division" << endl
                << "commonName = Common Name (eg, MD Root CA)" << endl
                << "commonName_max = 64" << endl
                << "commonName_default = Jarle Aase" << endl
                << "emailAddress = Email Address" << endl
                << "emailAddress_max = 40" << endl
                << "emailAddress_default = server@jgaa.com" << endl
                << "[ v3_ca ]" << endl
                << "basicConstraints = critical,CA:true" << endl
                << "subjectKeyIdentifier = hash" << endl
                << "[ v3_req ]" << endl
                << "nsCertType = server" << endl
                << "basicConstraints = critical,CA:false" << endl;
        }


        strstream cmd1;

        cout << "============================================" << endl
            << "Creating the server certificate." << endl;

        cmd1 << command
            << " req"
            << " -config " << resolve_path("server_cert.conf")
            << " -new "
            << " -keyout " << key_file
            << " -out newreq.pem"
            << (with_passwd_flag ? (const char *)"" : (const char *)" -nodes")
            << '\0';

        if (verbose_flag)
            clog << "Running command: " << cmd1.str() << endl;
        
        if (system(cmd1.str()) != 0)
        {
            clog << "Failed to create server certificate using command: "
                << cmd1.str();
            
            return -1;
        }
        
        if (!create_only_flag)
        {   
            strstream cmd2;

            cout << "============================================" << endl
            << "Signing the server certificate." << endl;
            
            cmd2 << command 
                << " ca "
                << " -config " << resolve_path("server_cert.conf")
                << " -policy policy_server"
                << " -out " << cert_file
                << " -infiles newreq.pem"
                << '\0';
            
            if (verbose_flag)
                clog << "Running command: " << cmd2.str() << endl;
            
            if (system(cmd2.str()) != 0)
            {
                clog << "Failed to sign server certificate using command: "
                    << cmd2.str();
                
                return -1;
            }
        }
    }

    if (cert_new_cli)
    {
        // Create new server certificate
        if (!is_file(resolve_path("client_cert.conf")))
        {
            ofstream conf_file(resolve_path("client_cert.conf").c_str());
            
            conf_file << "# Generated file" << endl
                << "[ ca ]" << endl
                << "default_ca	= CA_default" << endl
                << "[ CA_default ]" << endl
                << "dir	= " << cert_dir << endl
                << "certs = $dir/certs" << endl
                << "crl_dir = $dir/crl" << endl
                << "database = $dir/index.txt" << endl
                << "new_certs_dir = $dir/newcerts" << endl
                << "certificate	= " << rootcert_file << endl
                << "serial = $dir/serial" << endl
                << "crl	= $dir/crl.pem" << endl
                << "private_key = $dir/private/cakey.pem" << endl
                << "RANDFILE = $dir/private/rand" << endl
                << "default_days = 3650" << endl
                << "default_crl_days = 30" << endl
                << "default_md = md5" << endl
                << "preserve = no" << endl
                << "policy = policy_match" << endl
                << "# For the CA policy" << endl
                << "[ policy_match ]" << endl
                << "countryName = match" << endl
                << "stateOrProvinceName = match" << endl
                << "organizationName = match" << endl
                << "organizationalUnitName = optional" << endl
                << "commonName = supplied" << endl
                << "emailAddress = optional" << endl
                << "# For the 'anything' policy" << endl
                << "# At this point in time, you must list all acceptable 'object'" << endl
                << "# types." << endl
                << "[ policy_anything ]" << endl
                << "countryName = optional" << endl
                << "stateOrProvinceName = optional" << endl
                << "localityName = optional" << endl
                << "organizationName = optional" << endl
                << "organizationalUnitName	= optional" << endl
                << "commonName = supplied" << endl
                << "emailAddress = optional" << endl
                << "[ policy_client ]" << endl
                << "countryName = supplied" << endl
                << "stateOrProvinceName = optional" << endl
                << "localityName = optional" << endl
                << "organizationName = supplied" << endl
                << "organizationalUnitName	= optional" << endl
                << "commonName = supplied" << endl
                << "emailAddress = supplied" << endl
                << "[ req ]" << endl
                << "default_bits = 1024" << endl
                << "default_keyfile = ssft_server.key" << endl
                << "distinguished_name = req_distinguished_name" << endl
                << "string_mask = nombstr" << endl
                << "req_extensions = v3_req" << endl
                << "[ req_distinguished_name ]" << endl
                << "countryName = Country Name (2 letter code)" << endl
                << "countryName_default = NO" << endl
                << "endlcountryName_min = 2" << endl
                << "countryName_max = 2" << endl
                << "stateOrProvinceName = State or Province Name (full name)" << endl
                << "stateOrProvinceName_default = Hordaland" << endl
                << "localityName = Locality Name (eg, city)" << endl
                << "localityName_default = Bergen" << endl
                << "0.organizationName = Organization Name (eg, company)" << endl
                << "0.organizationName_default = Jgaas Internet" << endl
                << "organizationalUnitName = Organizational Unit Name (eg, section)" << endl
                << "organizationalUnitName_default = Certification Services Division" << endl
                << "commonName = Common Name (eg, MD Root CA)" << endl
                << "commonName_max = 64" << endl
                << "commonName_default = Jarle Aase" << endl
                << "emailAddress = Email Address" << endl
                << "emailAddress_max = 40" << endl
                << "emailAddress_default = client@jgaa.com" << endl
                << "[ v3_ca ]" << endl
                << "basicConstraints = critical,CA:true" << endl
                << "subjectKeyIdentifier = hash" << endl
                << "[ v3_req ]" << endl
                << "nsCertType = client" << endl
                << "basicConstraints = critical,CA:false" << endl;
        }

        if (1 == cert_new_cli)
        {
            strstream cmd1;

            cout << "============================================" << endl
            << "Creating the client certificate." << endl;
            
            cmd1 << command
                << " req"
                << " -config " << resolve_path("client_cert.conf")
                << " -new "
                << " -keyout " << use_client_key
                << " -out newreq.pem"
                << (with_passwd_flag ? (const char *)"" : (const char *)" -nodes")
                << '\0';
            
            if (verbose_flag)
                clog << "Running command: " << cmd1.str() << endl;
            
            if (system(cmd1.str()) != 0)
            {
                clog << "Failed to create client certificate using command: "
                    << cmd1.str();
                
                return -1;
            }
        }

        if (!create_only_flag)
        {
            strstream cmd2;

            cout << "============================================" << endl
            << "Signing the client certificate." << endl;
            
            cmd2 << command 
                << " ca "
                << " -config " << resolve_path("client_cert.conf")
                << " -policy policy_client"
                << " -out " << use_client_cert
                << " -infiles " << new_cert_name
                << '\0';
            
            if (verbose_flag)
                clog << "Running command: " << cmd2.str() << endl;
            
            if (system(cmd2.str()) != 0)
            {
                clog << "Failed to sign client certificate using command: "
                    << cmd2.str();
                
                return -1;
            }
        }
    }


    return 0;
}

