/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/


#include "StdAfx.h"
#include "ssft.h"

using namespace std;


ProgramLog::ProgramLog() :
mDoConsoleLog(true)
{    
}

ProgramLog::~ProgramLog()
{
    if (log_file)
        log_file.close();
}


ProgramLog& operator << (ProgramLog& o, const LogE& v)
{
    if (LOG_DATE == v)
    {
        if (log_file)
            log_file << get_time() << ' ';
    }
    else if (ENDL == v)
    {
        if (log_file)
            log_file << LOG_ENDL;
        
        if (o.mDoConsoleLog)
            std::cout << std::endl;
    }

    return o;
}

ProgramLog& operator << (ProgramLog& o, const SslError& v)
{
    return o << v.GetErrorString().c_str();
}

ProgramLog& operator << (ProgramLog& o, const sockaddr_in& v)
{
    return o << inet_ntoa(v.sin_addr) 
        << ":" 
        << ntohs(v.sin_port);
}

ProgramLog& operator << (ProgramLog& o, const SsftCmdE& v)
{
    switch(v)
    {
    case SSFT_CMD_QUIT:
        o << "SSFT_CMD_QUIT";
        break;
    case SSFT_CMD_TRANSF_CLI_TO_SEVER:
        o << "SSFT_CMD_TRANSF_CLI_TO_SEVER";
        break;
    case SSFT_CMD_TRANSF_SEVER_TO_CLI:
        o << "SSFT_CMD_TRANSF_SEVER_TO_CLI";
        break;
    case SSFT_CMD_DIR_SEVER_TO_CLI:
        o << "SSFT_CMD_DIR_SEVER_TO_CLI";
        break;
    case SSFT_CMD_DELETE_FILE_OR_DIR:
        o << "SSFT_CMD_DELETE_FILE_OR_DIR";
        break;
    case SSFT_CMD_CREATE_DIR:
        o << "SSFT_CMD_CREATE_DIR";
        break;
    default:
        return o << "(command: " << (int)v << ") ";
    }

    return o;
}

ProgramLog& operator << (ProgramLog& o, const SsftResultCodesE& v)
{
    switch(v)
    {
    case SSFT_RESULT_OPERATION_OK:
        return o << "SSFT_RESULT_OPERATION_OK";
    case SSFT_RESULT_ACCESS_DENIED:
        return o << "SSFT_RESULT_ACCESS_DENIED";
    case SSFT_RESULT_UNSPECIFIED_ERROR:
        return o << "SSFT_RESULT_UNSPECIFIED_ERROR";
    case SSFT_RESULT_BAD_PROTOCOL:
        return o << "SSFT_RESULT_BAD_PROTOCOL";
    case SSFT_RESULT_UNKNOWN_COMMAND:
        return o << "SSFT_RESULT_UNKNOWN_COMMAND";
    case SSFT_RESULT_HEADER_TOO_LONG:
        return o << "SSFT_RESULT_HEADER_TOO_LONG";
    case SSFT_RESULT_INVALID_BLEN:
        return o << "SSFT_RESULT_INVALID_BLEN";
    case SSFT_RESULT_NETWORK_ERROR:
        return o << "SSFT_RESULT_NETWORK_ERROR";
    case SSFT_RESULT_HANDSHAKE_FAILED:
        return o << "SSFT_RESULT_HANDSHAKE_FAILED";
    case SSFT_RESULT_FAILED_TO_OPEN_FILE:
        return o << "SSFT_RESULT_FAILED_TO_OPEN_FILE";
    case SSFT_RESULT_WRITE_ERROR:
        return o << "SSFT_RESULT_WRITE_ERROR";
    case SSFT_RESULT_READ_ERROR:
        return o << "SSFT_RESULT_READ_ERROR";
    case SSFT_RESULT_FILE_TOO_LONG:
        return o << "SSFT_RESULT_FILE_TOO_LONG";
    case SSFT_RESULT_NO_CERT_RECEIVED:
        return o << "SSFT_RESULT_NO_CERT_RECEIVED";
    case SSFT_RESULT_INVALID_REPLY:
        return o << "SSFT_RESULT_INVALID_REPLY";
    case SSFT_RESULT_INVALID_HEADER:
        return o << "SSFT_RESULT_INVALID_HEADER";
    case SSFT_RESULT_NOT_A_DIRECTORY:
        return o << "SSFT_RESULT_NOT_A_DIRECTORY";
    case SSFT_RESULT_UNKNOWN_HASH_DIGEST:
        return o << "SSFT_RESULT_UNKNOWN_HASH_DIGEST";
    case SSFT_RESULT_PATH_NOT_FOUND:
        return o << "SSFT_RESULT_PATH_NOT_FOUND";
    case SSFT_RESULT_SHARING_ERROR:
        return o << "SSFT_RESULT_SHARING_ERROR";
    default:
        return o << "(code: " << (int)v << ") ";
    }
}

#ifdef WIN32
ProgramLog& operator << (ProgramLog& o, const __int64& v)
{
    char buf[128];

    sprintf(buf, "%I64d", v);
    return o << buf;
}
#endif // WIN32
