/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/


class ofstream_shared
{
public:
    ofstream_shared()
        : mHandle(INVALID_HANDLE_VALUE)
    {
    }

    ~ofstream_shared()
    {
        close();
    }

    operator bool()
    {
        return is_open();
    }

    void open(LPCSTR name, int mode)
    {
        if (is_open())
            close();

        mHandle = ::CreateFile(name, 
            GENERIC_WRITE, 
            FILE_SHARE_READ, 
            NULL,
            OPEN_ALWAYS, 
            FILE_FLAG_SEQUENTIAL_SCAN, 
            NULL);
        
        if (is_open())
            ::SetFilePointer(mHandle, 0, 0, FILE_END);
    }

    bool is_open()
    {
        return mHandle != INVALID_HANDLE_VALUE;
    }

    void close()
    {
        if (is_open())
        {
            ::CloseHandle(mHandle);
            mHandle = INVALID_HANDLE_VALUE;
        }
    }

    void write(LPCSTR out, size_t bytes)
    {
        DWORD bytes_written = 0;
        ::WriteFile(mHandle, out, bytes, &bytes_written, NULL);
    }

private:
    HANDLE mHandle;
};

inline ofstream_shared& operator << (ofstream_shared& o, int v)
{
    char buf[32];
    int len = sprintf(buf, "%u", v);
    o.write(buf, len);

    return o;
}

inline ofstream_shared& operator << (ofstream_shared& o, unsigned int v)
{
    char buf[32];
    int len = sprintf(buf, "%d", v);
    o.write(buf, len);

    return o;
}

inline ofstream_shared& operator << (ofstream_shared& o, long v)
{
    char buf[32];
    int len = sprintf(buf, "%ld", v);
    o.write(buf, len);

    return o;
}

inline ofstream_shared& operator << (ofstream_shared& o, unsigned short v)
{
    char buf[32];
    int len = sprintf(buf, "%u", (unsigned)v);
    o.write(buf, len);

    return o;
}

inline ofstream_shared& operator << (ofstream_shared& o, LPCSTR v)
{
    o.write(v, strlen(v));

    return o;
}

inline ofstream_shared& operator << (ofstream_shared& o, 
                              const std::string& v)
{
    o.write(v.c_str(), v.size());

    return o;
}

inline ofstream_shared& operator << (ofstream_shared& o, char v)
{
    o.write(&v, 1);

    return o;
}



