/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/



#include "StdAfx.h"
#include "getopt.h"
#include <stdio.h>
#include <string.h>

const char *optarg;
int optind, opterr = 1, optopt;

static int getopt_err(int ch, int arg, const char *parg)
{
    char varg[2];
    optopt = arg;

    if (!parg)
    {
        varg[0] = arg;
        varg[1] = 0;
        parg = varg;
    }

    if (opterr) switch (ch)
    {
        case ':':
            fprintf(stderr, "Missing argument for %s\n", parg);
            break;
        case '?':
            fprintf(stderr, "Unknown argument %s\n", parg);
            break;
        default:
            fprintf(stderr, "getopt_err(): internal error\n");
    }

    return ch;
}


int getopt_long(int argc,
                char * const argv[],
                const char *optstring,
                const struct option *longopts,
                int *longindex)
{

    static int opt_idx;
    char target = 0;
    const char *p_arg = NULL, *eq_p = NULL, *p_target = NULL;
    unsigned eq_pos = 0;
    int opt_longindex, *p_flag;

    if (!opt_idx)
    {
        if (++optind >= argc)
        {
            optind = argc;
            return -1; // End of arguments
        }

    }

    if (opt_idx || ((argv[optind][0] == '-') && (argv[optind][1] != '-')))
    {
        // Short form
        target = argv[optind][++opt_idx];
        if (target == 0)
        {
            opt_idx = 0;
            return getopt_long(argc, argv, optstring, longopts, longindex);
        }

        p_arg = strchr(optstring, target);
        if (!p_arg)
            return getopt_err('?', target, NULL);

        if (*++p_arg == ':')
        {
            optarg = &argv[optind][opt_idx + 1];
            opt_idx = 0;
            if (!*optarg)
                optarg = NULL;
            if (!optarg)
            {
                if (*++p_arg == ':')
                {
                    // Ok. Optional argument.
                    goto have_target;
                }

                if (++optind >= argc)
                {
                    --optind;
                    target = ':';
                    goto have_target;
                }

                optarg = argv[optind];
                goto have_target;
            }
        }

        if (optarg && !*optarg)
            optarg = NULL;

have_target:
        return target;
    }

    if (((argv[optind][0] == '-') && (argv[optind][1] == '-')))
    {
        // Long form

        if (!*(p_target = &argv[optind][2]))
            return -1; /* "--" argument ends parameter parsing */

        eq_p = strchr(p_target, '=');
        eq_pos = eq_p ? eq_p - p_target : strlen(p_target);

        // Search for option
        for(opt_longindex = 0; longopts[opt_longindex].name ; opt_longindex++)
        {
            if ((strlen(longopts[opt_longindex].name) >= eq_pos)
                    && (longopts[opt_longindex].has_arg
                        ? !strncmp(longopts[opt_longindex].name, p_target, eq_pos)
                        : !strcmp(longopts[opt_longindex].name, p_target)))
            {

                if (longindex)
                    *longindex = opt_longindex;

                if (longopts[opt_longindex].has_arg)
                {
                    if (eq_p)
                    {
                        optarg = ++eq_p;
                        if (!*optarg)
                            return getopt_err(':', 0, longopts[opt_longindex].name);
                    }
                    else if (longopts[opt_longindex].has_arg == required_argument)
                    {
                        if ((++optind >= argc)
                                || (!*(optarg = argv[optind])))
                        {
                            --optind;
                            return getopt_err(':', 0, longopts[opt_longindex].name);
                        }
                    }
                }

                if (longopts[opt_longindex].flag == NULL)
                    return longopts[opt_longindex].val;

                p_flag = longopts[opt_longindex].flag;
                *p_flag = longopts[opt_longindex].val;
                return 0;
            }
        }

        return getopt_err('?', 0, p_target);
    }

    // Not a formal parameter
    return -1;
}
