/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

// Debug strings
// Server
// --debug --verbose --server --cert-dir=..\certs -c ..\svr_test\ssft.conf
// --server --port=231 --conf-file=C:\WINNT\ssftbackup.conf 
//
// Client
// --verbose --debug --cert-dir=..\certs test.zip localhost:/
// --verbose --debug --cert-dir=..\certs -C dir localhost:/
// --verbose --debug --cert-dir=..\certs -R -C replicate C:\ssft_test\local localhost:/


#include "StdAfx.h"
#include "ssft.h"
#include <openssl/evp.h>

using namespace std;

int my_port = 0;
long my_host;
int verbose_flag = 0;
int recurse_flag = 0;
int debug_flag = 0;
int nocert_flag = 0;
int server_mode = 0;
int daemon_mode = 0;
int alarm_delay = 10;
int delete_flag = 0;
char *myname = "";
std::string hostname;
std::string logfile_name;
int cert_new_ca = 0, cert_new_cli = 0, cert_new_svr = 0;
int with_passwd_flag;
int create_only_flag;
std::string cert_file;
std::string key_file;
std::string cli_cert_file;
std::string cli_key_file;
std::string rootcert_file;
std::string cert_dir;
std::string config_file;
std::string pid_file;
std::string hash_digest; // Hash digest used to verify filestamps
std::string date_fmt = "%d %b %Y %H:%M:%S";
std::string sign_cert;
bool do_continue = true;
ProgramLog log;
userptr_list_t user_list;
SsftClientModeE curr_cmd = SSFT_MODE_COPY;

#ifdef WIN32
int reply_delay = 0;
const char *service_name = "ssft";
#endif

#ifdef WIN32
    ofstream_shared log_file;
#else
    ofstream log_file;
#endif


struct sockaddr socaddr;
struct sockaddr_in *pin = (struct sockaddr_in *)&socaddr;
int sockaddr_len = sizeof(sockaddr);

class CleanUp
{
public:
    CleanUp()
    {}
    ~CleanUp()
    {
        if (!pid_file.empty())
            unlink(pid_file.c_str());
    }
};

void brief_usage()
{
    cerr << myname << " --help for help." << endl;
    exit(-1);
}

void usage()
{
    cerr << "Client mode:" << endl;
    cerr << ' ' << myname << " [options] host:source target" << endl;
    cerr << ' ' << myname << " [options] source [source...] host:target" << endl;
    cerr << ' ' << myname << " [options] --command dir host:target" << endl;
    cerr << ' ' << myname << " [options] --command delete host:target" << endl;
    cerr << ' ' << myname << " [options] --command mkdir host:target" << endl;
    cerr << ' ' << myname << " [options] -C replicate [host:]source [host:]target" << endl;
    cerr << "(--command can be substituted with -C)" << endl;
    cerr << "Server mode:" << endl;
    cerr << ' ' << myname << " [options] [hostname]" << endl;
    cerr << "Utility mode (install, uninstall, certificate management):" << endl;
    cerr << ' ' << myname << " option [...]" << endl;
    cerr << "Options: " << endl;
    cerr << " -c  --conf-file=name   Specifies the configuration file to use" << endl;
    cerr << " -p  --port=#           Set port number. Default is 230" << endl;
    cerr << " -v  --version          Show version number and quit" << endl;
    cerr << "     --verbose          Turn on verbose mode" << endl;
    cerr << "     --debug            Turn on debug messages" << endl;
    cerr << "     --server           Server mode (default is client mode)" << endl;
    cerr << "     --daemon           Runs as a daemon" << endl;
    cerr << "     --delete           Delete orphan files during replication." << endl;
    cerr << "                        If used with --replicate, orphan files are deleted." << endl;
    cerr << " -R  --recurse          Enable recursive dir-listings and repl." << endl;
    cerr << "     --hash=sha1|md5    Hash digest to use in dir-listings and repl." << endl;
    cerr << " -l  --log-file=name    Name of logfile" << endl;
    cerr << " -h  --help             Show this help" << endl;
    cerr << "     --cert-dir=path    Certificate directory" << endl;
    cerr << "     --root-cert=name   Root certificate file" << endl;
    cerr << "     --client-cert=name Client certificate" << endl;
    cerr << "     --client-key=name  Client private key file" << endl;
    cerr << "     --server-cert=name Server certificate" << endl;
    cerr << "     --server-key=name  Server private key file" << endl;
    //cerr << "     --without-cert     Don't use client certificate" << endl;
    cerr << "     --pid-file=name    Write process ID to file" << endl;

#ifdef WIN32
    cerr << " -s  --service-name=name Sets the service name. Used with --install" << endl;
    cerr << " -i  --install          Install as a NT service (and quit)" << endl;
    cerr << " -u  --uninstall        Uninstall as a NT service (and quit)" << endl;
#endif

    cerr << " -L  --license          Show licensing terms (GPL)" << endl;

    cerr << endl;
    cerr << "Certificate management:" << endl;
    cerr << "     --generate-ca      Generates a root certificate" << endl;
    cerr << "     --new-server-cert  Generates a server certificate" << endl;
    cerr << "     --new-client-cert  Generates a client certificate" << endl;
    cerr << "     --sign-cert=name   Sign the certificate 'name'." << endl;
    cerr << "Options:" << endl;
    cerr << "     --with-password    Create a certificate with password" << endl;
    cerr << "     --create-only      Only create a cert. Do not sign." << endl;
    exit(-1);
}

#ifdef WIN32

BOOL WINAPI win32_break(DWORD dwCtrlType)
{
    log << LOG_DATE << "**** Signalled ***** ";

    switch(dwCtrlType)
    {
    case CTRL_LOGOFF_EVENT:
        log << "CTRL_LOGOFF_EVENT - ";
daemon_override:
        if (!daemon_mode)
            goto shutdown;
        log << "Running in daemon mode. The event is ignored." << ENDL;
        break;
    case CTRL_CLOSE_EVENT:
        log << "CTRL_CLOSE_EVENT - ";
        goto daemon_override;
        break;
    case CTRL_BREAK_EVENT:
        log << "CTRL_BREAK_EVENT - ";
shutdown:
        if (!daemon_mode)
        {
            static int sig_cnt;
            if (sig_cnt++ > 0)
            {
                log << "OK! OK! Quitting now!" << ENDL;
                abort();
            }
        }
        log << "Shutting down." << ENDL;
        do_continue = false;
        break;
    case CTRL_C_EVENT:
        log << LOG_DATE << "CTRL_C_EVENT - ";
        goto shutdown;
        break;
    case CTRL_SHUTDOWN_EVENT:
        log  << "CTRL_SHUTDOWN_EVENT - ";
        goto shutdown;
        break;
    }

    return true;
}

#else
void sig_alarm(int sig)
{
    ;
}

void sig_break(int signal)
{
    if (!daemon_mode)
    {
        static int sig_cnt;
        if (sig_cnt++ > 0)
        {
            log << "OK! OK! Quitting now!" << ENDL;
            abort();
        }
    }
    log << LOG_DATE << "**** Interrupted. Shutting down" << ENDL;
    
    do_continue = false;
}

void sig_hup(int signal)
{
    log << LOG_DATE << "**** Reloading configuration file" << ENDL;
    
    config_reload();
}

#endif



int main(int argc, char* argv[])
{
    CleanUp my_cleanup;
    bool need_config = false;
#ifdef WIN32
    bool do_uninstall = false;
    bool do_install = false;
#else
#   define do_uninstall false
#   define do_install false
#endif
    char *p = argv[0];
    if (p)
    {
        while (*p)
        {
            if (ISSLASH(*p))
                myname = p + 1;
            ++p;
        }
    }
    
    if (!*myname)
        myname = argv[0];

    enum OptionE
    {
        OPT_CERT_DIR = 256,
        OPT_CERT_ROOT,
        OPT_CERT_CLI,
        OPT_KEY_CLI,
        OPT_CERT_SVR,
        OPT_KEY_SVR,
        OPT_PIDFILE,
        OPT_HASH,
        OPT_SIGN_CERT
    };
    
    struct option options[] = 
    {
        {"conf-file", required_argument, NULL, 'c'},
        {"verbose", no_argument, &verbose_flag, 1},
        {"version", no_argument, NULL, 'v'},
        {"install", no_argument, NULL, 'i'},
        {"uninstall", no_argument, NULL, 'u'},
        {"port", required_argument, NULL, 'p'},
        {"server", no_argument, &server_mode, 1},
        {"daemon", no_argument, &daemon_mode, 1},
        {"debug", no_argument, &debug_flag, 1},
        {"delete", no_argument, &delete_flag, 1},
        {"recurse", no_argument, NULL, 'R'},
        {"log-file", required_argument, NULL, 'l'},
        {"wait", required_argument, NULL, 'w'},
        {"timeout", required_argument, NULL, 't'},
        {"license", no_argument, NULL, 'L'},

        {"cert-dir", required_argument, NULL, OPT_CERT_DIR},
        {"root-cert", required_argument, NULL, OPT_CERT_ROOT},
        {"client-cert", required_argument, NULL, OPT_CERT_CLI},
        {"client-key", required_argument, NULL, OPT_KEY_CLI},
        {"server-cert", required_argument, NULL, OPT_CERT_SVR},
        {"server-key", required_argument, NULL, OPT_KEY_SVR},
        //{"without-cert", no_argument, &nocert_flag, 1},
        {"pid-file", required_argument, NULL, OPT_PIDFILE},
        {"command", required_argument, NULL, 'C'},
        {"hash", required_argument, NULL, OPT_HASH},

        {"generate-ca", no_argument, &cert_new_ca, 1},
        {"new-server-cert", no_argument, &cert_new_svr, 1},
        {"new-client-cert", no_argument, &cert_new_cli, 1},
        {"with-password", no_argument, &with_passwd_flag, 1},
        {"create-only", no_argument, &create_only_flag, 1},
        {"sign-cert", required_argument, NULL, OPT_SIGN_CERT},

#ifdef WIN32
        {"reply-delay", required_argument, NULL, 'r'},
        {"service-name", required_argument, NULL, 's'},
#endif

        {"help", no_argument, NULL, 'h'},
        { NULL, 0, NULL, 0}
    };
    
    int ch = 0;
    while((ch = getopt_long(argc, argv, "RLhvp:t:l:s:c:C:", options, NULL)) >= 0)
    {
        switch(ch)
        {
        case 'v':
            cerr << PROGRAM << " " << VERSION << endl;
            return 0;

        case 'h':
            usage();
            break;

        case 'R':
            recurse_flag = true;
            break;

        case 'c':
            config_file = optarg;
            need_config = true;
            break;
            
        case 'p':
            my_port = atoi(optarg);
            if (verbose_flag)
                cerr << "Setting port to " << my_port << endl;
            break;
            
#ifdef WIN32
        case 'i':
            do_install = true;
            break;

        case 'u':
            do_uninstall = true;
            break;
        case 's':
            service_name = optarg;
            break;
#endif

        case 'L':
            show_gpl();
            return 0;
            
        case 'l':
            if (verbose_flag)
                cerr << "Opening logfile " << optarg << endl;
            
            logfile_name = optarg;
            log_file.open(optarg, ios::out|ios::app);

            if (!log_file.is_open())
            {
                cerr << "Failed to open logfile " << optarg << endl;
                return -1;
            }
            else
            {
                log_file << "===================================================="
                    << SYSENDL
                    << "===================================================="
                    << SYSENDL;
            }
            break;

        case OPT_CERT_DIR:
            cert_dir = optarg;
            if (!is_dir(optarg))
            {
                log << "Not a directory \""
                    << optarg
                    << "\". Quitting.";
                
                return -1;
            }
            break;

        case OPT_SIGN_CERT:
            sign_cert = optarg;
            cert_new_cli = 2;
            break;
        
        case OPT_CERT_ROOT:
            rootcert_file = optarg;
            break;
        
        case OPT_CERT_CLI:
            cli_cert_file = optarg;
            break;

        case OPT_KEY_CLI:
            cli_key_file = optarg;
            break;

        case OPT_CERT_SVR:
            cert_file = optarg;
            break;

        case OPT_KEY_SVR:
            key_file = optarg;
            break;

        case OPT_PIDFILE:
            pid_file = optarg;
            break;

        case OPT_HASH:
            hash_digest = optarg;
            break;

        case 'C':
            {
                static const char *cmds[] = 
                {
                    {"copy"}, {"dir"}, {"replicate"}, {"delete"}, {"mkdir"}
                };

		int index;
                for(index = 0; index < SSFT_MODE_INVALID; index++)
                {
                    if (strcmp(cmds[index], optarg) == 0)
                    {
                        curr_cmd = (SsftClientModeE)index;
                        break;
                    }
                }

                if (SSFT_MODE_INVALID == index)
                {
                    cerr << "Unknown command:" << optarg << endl;
                    return -1;
                }
            }
            break;
            
        case 0:
            break;
            
        default:
            brief_usage();
        }
    }

    if (do_install && daemon_mode && !server_mode)
    {
        log << LOG_DATE << "Cannot be daemon in server-mode. Use --server as well!"
            << ENDL;
        return -1;
    }

    if (!(do_install || do_uninstall))
    {
        if (!config_reload() && (need_config || server_mode))
        {
            log << LOG_DATE
                << "Error: Failed to load configuration file \""
                << config_file
                << "\"."
                << ENDL;
            
            return -1;
        }
    }

    // Set default values if unset
    if (cert_file.empty())
        cert_file = "ssft_daemon.cert";
    if (key_file.empty())
        key_file = "ssft_daemon.key";
    if (cli_cert_file.empty())
        cli_cert_file = "ssft_client.cert";
    if (cli_key_file.empty())
        cli_key_file = "ssft_client.key";
    if (rootcert_file.empty())
        rootcert_file = "ssft_root.cert"; 
    if (!my_port)
        my_port = SSFT_PORT;

    cert_file = resolve_path(cert_file);
    key_file = resolve_path(key_file);
    cli_cert_file = resolve_path(cli_cert_file);
    cli_key_file = resolve_path(cli_key_file);
    rootcert_file = resolve_path(rootcert_file);

    // Check for special commands

    if (cert_new_ca || cert_new_cli || cert_new_svr)
        return do_certs();

    if (!pid_file.empty())
    {
        if (debug_flag)
            log << LOG_DATE << "Opening pid-file \"" << pid_file << "\" ";
        ofstream pidfile(pid_file.c_str());
        if (pidfile)
        {
            if (debug_flag)
                log << "OK" << ENDL;
            pidfile << getpid() << endl;
        }
        else
        {
            log << "failed. " << strerror(errno) << ENDL;
        }
    }
    
#ifdef WIN32
    WSADATA wsaData;
    if (WSAStartup(MAKEWORD(2, 0), &wsaData ) == -1)
    {
        log << LOG_DATE << "Failed to initialize winsock. " << GET_SOCKET_ERROR << ENDL;
        return -1;
    }

    SetConsoleCtrlHandler(win32_break, true);

#else

#if HAVE_OPENSSL_ADD_ALL_DIGESTS
    OpenSSL_add_all_digests();
#else
    SSLeay_add_all_digests();
#endif

    if (daemon_mode)
    {
	    if (verbose_flag)
	        log << LOG_DATE << "Switching to daemon mode" << ENDL;

        switch(fork())
        {
        case -1:
            log << LOG_DATE << "Failed to fork " << errno << ENDL;
            return -1;
        case 0: // child
            close(STDIN_FILENO);
            close(STDOUT_FILENO);
            close(STDERR_FILENO);
            if (setsid() == -1)
                return -1;
            if (fork() != 0)
                return 0;
            chdir("/");
            umask(0);
            freopen("/dev/null", "r", stdin);
            freopen("/dev/null", "w", stdout);
            freopen("/dev/console", "w", stderr);
            break;
        default:
            return 0;
        }
        
    }
    
    signal(SIGALRM, sig_alarm);
    signal(SIGQUIT, sig_break);
    signal(SIGINT, sig_break);
    signal(SIGHUP, sig_hup);
#endif // WIN32
    

#ifdef WIN32
    if (do_uninstall)
    {
        MyNtService my_service;
        if (!my_service.IsInstalled())
        {
            log << LOG_DATE << "The application is not installed as a NT service!" << ENDL;
            return -1;
        }
        if (my_service.Uninstall())
        {
            log << LOG_DATE << "Successfully uninstalled as NT service" << ENDL;
            if (!do_install)
                return 0;
        }
        else
        {
            log << LOG_DATE << "Failed to uninstall as NT servive!" << ENDL;
            return -1;
        }
    }

    if (do_install)
    {
        string myargs;
        MyNtService my_service;
        if (my_service.IsInstalled())
        {
            log << LOG_DATE << "The application is already installed." << ENDL;
            return -1;
        }
        
        char buffer[32];
        
        myargs += "--daemon --server ";


        if (SSFT_PORT != my_port)
        {
            myargs += "--port=";
            myargs += itoa(my_port, buffer, 10);
            myargs += ' ';
        }
        
        if (!logfile_name.empty())
        {
            myargs += "\"--log-file=";
            myargs += logfile_name;
            myargs += "\" ";
        }

        if (!config_file.empty())
        {
            myargs += "\"--conf-file=";
            myargs += config_file;
            myargs += "\" ";
        }

        if (!cert_dir.empty())
        {
            myargs += "\"--cert-dir=";
            myargs += cert_dir;
            myargs += "\" ";
        }

        if (!pid_file.empty())
        {
            myargs += "\"--fid-file=";
            myargs += pid_file;
            myargs += "\" ";
        }
        
        if (hostname.empty())
            myargs += hostname;
        
        if (my_service.Install(myargs.c_str()))
        {
            log << LOG_DATE << "Successfully installed as a NT service as \"" 
                << service_name
                << "\". "
                << ENDL
                << "You must now enter the service applet in the "
                "control panel and configure the startup mode."
                << ENDL;
            return 0;
        }
        {
            log << LOG_DATE << "Failed to install as NT servive!" << ENDL;
            return -1;
        }
    }

    if (daemon_mode)
    {
        MyNtService my_service;
        if (!my_service.StartService())
        {
            log << LOG_DATE << "Cannot start as NT/Windows 2000 service!" << ENDL;
            return -1;
        }

        return 0;
    }
#endif // WIN32
    
    return do_process(argc - optind, &argv[optind]);
}

int do_process(int argc, char **argv)
{
  if (verbose_flag)
    {
    log << LOG_DATE 
        << PROGRAM 
        << " version " 
        << VERSION
        << ", Copyright (C) 2001 - 2002 by Jarle (jgaa) Aase" 
        << ENDL
        << PROGRAM 
        << " comes with ABSOLUTELY NO WARRANTY"
        << ENDL
        << "This is free software, and you are welcome to redistribute it "
        << ENDL
        << "under certain conditions;"
        << ENDL
        << myname << " --license for details."
        << ENDL;
    }

    if (server_mode)
    {
        if (1 == argc)
            hostname = argv[0];

        return do_server();
    }
    
    return do_client_xmit(argc, argv);
}
