/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/


#ifndef SSH_INCLUDES_H
#define SSH_INCLUDES_H

#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>

#ifdef WIN32

#   include "winsock2.h"
#   include "MyNtService.h"
#   include <io.h>
#   include <direct.h>
#   include <process.h>

typedef __int64 ssft_flen_t;
typedef __int64 ssft_date_t;

#else

#   include <unistd.h>
#   include <sys/socket.h>
#   include <netinet/in.h>
#   include <sys/time.h>
#   include <netinet/tcp.h>
#   include <arpa/inet.h>
#   include <netdb.h>
#   include <errno.h>
#   include <stdlib.h>
#   include <signal.h>
#   include <fcntl.h>
#   include <dirent.h>

typedef long long ssft_flen_t;
typedef long long ssft_date_t;

#endif

#include <iostream>
#include <strstream> 
#include <fstream>
#include <string>
#include <vector>
#include <list>
#include <time.h>
#include <exception>
#include <set>

#if HAVE_CONFIG_H
#   include <rsa.h>       /* SSLeay stuff */
#   include <crypto.h>
#   include <x509.h>
#   include <pem.h>
#   include <ssl.h>
#   include <err.h>
#   include <evp.h>
#else
#   include <openssl/rsa.h>       /* SSLeay stuff */
#   include <openssl/crypto.h>
#   include <openssl/x509.h>
#   include <openssl/pem.h>
#   include <openssl/ssl.h>
#   include <openssl/err.h>
#   include <openssl/evp.h>
#endif

#if HAVE_STAT64
#   define ssft_stat    stat64
#   define ssft_fstat   fstat64
    typedef struct stat64 ssft_stat_t;
#elif defined(WIN32)
#   define ssft_stat    _stati64
#   define ssft_fstat   _fstati64
    typedef struct _stati64 ssft_stat_t;
#else
#   define ssft_stat    stat
#   define ssft_fstat   fstat
    typedef struct stat ssft_stat_t;
#endif

#if HAVE_OPEN64
#   define ssft_open    open64
#else
#   define ssft_open    open
#endif

#ifdef HAVE_GETOPT_LONG
#   include <getopt.h>
#else
#   include "ports/getopt.h"
#endif

#ifdef WIN32
/* Microsoft is working very hard to push all Wintendo developers 
   over to VisualBasic. One of the methods is to make the
   "standard" API's worthless, so that developers have to write
   as much as possible from scratch if they choose to use C++ or C.
   std::ofstream is implemented without any way to turn on file-sharing,
   so that the stream classes are useless for any logging functions.
   We have to write our own class...
*/
#   include "ofstream_shared.h"

    extern  ofstream_shared log_file;
#   define LOG_ENDL "\r\n"
#else
    extern ofstream log_file;
#   define LOG_ENDL std::endl
#endif

#ifdef WIN32
#   define SYSSLASH '\\'
#   define ISSLASH(a) (a == '\\')
#   define GET_SOCKET_ERROR WSAGetLastError()
#   define EXPLAIN_SOCKET_ERROR GET_SOCKET_ERROR
#   define EWOULDBLOCK  WSAEWOULDBLOCK
#   define HAVE_STRICMP 1
#   define HAVE_STRNICMP 1
#   define HAVE_ERR_ERROR_STRING_N 1
typedef int socklen_t;
#else
#   define SYSSLASH '/'
#   define ISSLASH(a) (a == '/')
#   define GET_SOCKET_ERROR errno
#   define EXPLAIN_SOCKET_ERROR strerror(errno)
#   define SOCKET int
#   define BOOL int
#   define closesocket(a) close(a)
#endif


#endif // SSH_INCLUDES_H

