/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#include "StdAfx.h"
#include "ssft.h"
#include <assert.h>
#include <malloc.h>

#ifndef WIN32
#   include <grp.h>
#   include <pwd.h>
#endif


using namespace std;

char *get_time()
{
    static char buf[64];
    time_t now = time(NULL);
    struct tm *ptm = localtime(&now);
    if (!ptm)
        return "***notime***";
    
    strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M", ptm);
    
    return buf;
}

int set_async_mode(SOCKET sck, bool makeAsync)
{    
    
#if WIN32
    unsigned long sck_mode = makeAsync;
    if (::ioctlsocket(sck, FIONBIO, &sck_mode))
        return -1;
#else
    int flags = 0;
    
    if ((flags = ::fcntl(sck, F_GETFL, 0)) != 0)
        return -1;
    
    if (makeAsync)
        flags |= O_NONBLOCK;
    else
        flags &= ~O_NONBLOCK;
    
    if (fcntl(sck, F_SETFL, flags) != 0)
        return -1;
#endif

    return 0;
}

void sleep(int seconds)
{
#ifdef WIN32
    Sleep(seconds * 1000);
#else
    alarm(seconds);
    pause();
#endif
}

SOCKET create_socket(bool doBind)
{
    SOCKET sck = socket(AF_INET, SOCK_STREAM, 0);
    if (sck == -1)
    {
        log << LOG_DATE << "Failed to create socket. " << GET_SOCKET_ERROR << ENDL;
        return -1;
    }
    
    pin->sin_port = htons(my_port);
    pin->sin_addr.s_addr = my_host;
    pin->sin_family = AF_INET;
    
    if (doBind)
    {
        if (bind(sck, &socaddr, sockaddr_len) != 0)
        {
            log << LOG_DATE << "Failed to bind socket. " << EXPLAIN_SOCKET_ERROR << ENDL;
            return -1;
        }
    }
    
    return sck;
}

bool resolve_hostname()
{
    my_host = inet_addr(hostname.c_str());
    if (((SOCKET)INADDR_NONE) == my_host)
    {
        if (verbose_flag)
        {
            log << LOG_DATE << "Looking up hostname " << hostname << " ... ";
            cout.flush();
        }
        
        struct hostent *pent = gethostbyname(hostname.c_str());
        
        if (NULL == pent)
        {
            if (verbose_flag)
                log << "FAILED" << ENDL;
            
            log << LOG_DATE << optarg << ": Unknown my_host" << ENDL;
            return false;
        }
        
        my_host = *(long *)pent->h_addr;
        
        if (verbose_flag)
        {
            in_addr my_addr;
            my_addr.s_addr = my_host;
            log << inet_ntoa(my_addr) << ENDL;
        }
    }

    return true;
}

bool is_dir(const char *path)
{
    ssft_stat_t st;
    string my_path = path;

    if (my_path.empty())
        return false;

    if (IS_SLASH(my_path[my_path.size() -1]))
        my_path.resize(my_path.size() -1);

    if (ssft_stat(my_path.c_str(), &st))
        return false;

    if ((st.st_mode & S_IFDIR) != S_IFDIR)
        return false;

    return true;
}

bool is_dir(const std::string& path)
{
    return is_dir(path.c_str());
}

bool is_file(const char *path)
{
    ssft_stat_t st;

    if (ssft_stat(path, &st))
        return false;

    if ((st.st_mode & S_IFREG) != S_IFREG)
        return false;

    return true;
}

bool is_file(const std::string& path)
{
    return is_file(path.c_str());
}

std::string resolve_path(const std::string& name)
{
    if (cert_dir.empty())
        return name;

    string my_name = name;
    const char *path_str = my_name.c_str();

#ifdef WIN32
    int i;
    while((i = my_name.find('\\')) != my_name.npos)
        my_name[i] = '/';
#endif

    if (path_str && *path_str && ('/' == *path_str))
        return name;

#ifdef WIN32
    if (*path_str)
    {
        if (isalpha(*path_str)
            && (':' == *++path_str))
            return name;
    }
#endif

    string path = cert_dir;

    if (!path.empty() && ('/' != path[path.size() -1]))
        path += '/';

    //cout << "cert_dir=" << cert_dir << ", name=" << name << endl;

    path += name;

#ifdef WIN32
    while((i = my_name.find('\\')) != my_name.npos)
        my_name[i] = '/';
#endif

    return path;
}

ssft_flen_t htonll(ssft_flen_t v)
{
    return 
#ifdef WIN32
     ((((v) & 0xff00000000000000) >> 56) 
      | (((v) & 0x00ff000000000000) >> 40)
      | (((v) & 0x0000ff0000000000) >> 24)
      | (((v) & 0x000000ff00000000) >> 8)
      | (((v) & 0x00000000ff000000) << 8)
      | (((v) & 0x0000000000ff0000) << 24)
      | (((v) & 0x000000000000ff00) << 40)
      | (((v) & 0x00000000000000ff) << 56));
#else
     ((((v) & 0xff00000000000000ull) >> 56) 
      | (((v) & 0x00ff000000000000ull) >> 40)
      | (((v) & 0x0000ff0000000000ull) >> 24)
      | (((v) & 0x000000ff00000000ull) >> 8)
      | (((v) & 0x00000000ff000000ull) << 8)
      | (((v) & 0x0000000000ff0000ull) << 24)
      | (((v) & 0x000000000000ff00ull) << 40)
      | (((v) & 0x00000000000000ffull) << 56));
#endif
}

std::string merge_paths(const std::string& v1,
                        const std::string& v2,
                        char slash_ch)
{
    string new_path = with_slash(v1, slash_ch);
    const char *p = v2.c_str();
    if (p)
    {
        if (IS_SLASH(*p))
            ++p;

        if (*p)
            new_path += p;
    }
    
    return new_path;
}

// Strip file-name from pathname
std::string get_pathname(const std::string& filePath)
{
    std::string my_path = filePath;

    const char *rval = my_path.c_str();
    char *p = &my_path[0], *last_slash  = NULL;

    for(; *p; p++)
    {
        if (IS_SLASH(*p))
            last_slash = p;
    }

    if (last_slash)
    {
#ifdef WIN32
        // Check for "X:\"
        if ((last_slash == &my_path[3])
            && isalpha(my_path[0])
            && (':' == my_path[2]))
        {
            return filePath;
        }

        *last_slash = 0;
#endif
    }

    return rval;
}

std::string with_slash(const std::string& v, 
                       char slash_ch)
{
    if (v.empty())
    {
        char my_slash[2];
        my_slash[0] = slash_ch;
        my_slash[1] = '\0';
        return my_slash;
    }
    else
    {
        char ch = v[v.size() -1];
        
#ifdef WIN32 
        if ('/' == ch)
        {
            string rval = v;
            rval[rval.size() -1] = slash_ch;
            return rval;
        }
#endif // WIN32 

        if (ch != SYSSLASH)
        {
            string rval = v;
            rval += slash_ch;
            return rval;
        }
    }
    
    return v;
}

bool set_ownership(const std::string& path, 
                   int unix_mode,
                   const char *user, 
                   const char *group)
{
    bool rval = true;

    if (debug_flag)
    {
      log << LOG_DATE << "(debug): set_ownership("
	  << path << ','
	  << unix_mode << ','
	  << (user ? user : "(NULL)") << ','
	  << (group ? group : "(NULL)")
	  << ") called"
	  << ENDL;
    }

    if (unix_mode)
    {
        if (chmod(path.c_str(), unix_mode) != 0)
        {
            log << LOG_DATE
                << "Warning: Failed to chmod the path \""
                << path
                << "\" to mode (as decimal): "
                << unix_mode
                << " due to error "
                << strerror(errno)
                << ENDL;
            rval = false;
        }
    }
    
    if ((user && *user) || (group && *group))
    {
#ifdef WIN32
#else
        // UNIX
        ssft_stat_t st;
        if (ssft_stat(path.c_str(), &st) == 0)
        {
            if (user)
            {
                struct passwd *p = getpwnam(user);
                if (p)
                {
                    st.st_uid = p->pw_uid;
                    st.st_gid = p->pw_gid;
                }
                else
                {
                    log << LOG_DATE
                        << "Warning: Failed to lookup user \""
                        << user
                        << " due to error "
                        << strerror(errno)
                        << ENDL;
                    rval = false;
                }
            }
            
            if (group)
            {
                struct group *p = getgrnam(group);
                if (p)
                {
                    st.st_gid = p->gr_gid;
                }
                else
                {
                    log << LOG_DATE
                        << "Warning: Failed to lookup group \""
                        << group
                        << " due to error "
                        << strerror(errno)
                        << ENDL;
                    rval = false;
                }
            }
            
            if (chown(path.c_str(), st.st_uid, st.st_gid) != 0)
            {
                log << LOG_DATE
                    << "Warning: Failed to chown \""
                    << path
                    << "\" to "
                    << ((user && *user) ? user : "-") 
                    << ":" 
                    << ((group && *group) ? group : "-")
                    << " due to error "
                    << strerror(errno)
                    << ENDL;
                rval = false;
            }
        }
        else
        {
            log << LOG_DATE
                << "Warning: Failed to stat the path \""
                << path
                << " due to error "
                << strerror(errno)
                << ENDL;
            rval = false;
                rval = false;
        }
#endif
    }

    return rval;
}

bool make_dirs(const std::string& path,
               int unix_mode,
               const char *user, 
               const char *group)
{
    string my_path;
    const char *p = path.c_str();
    bool virgin = true;

    if (!p || !*p)
        return false;

    while(*p)
    {
        if (IS_SLASH(*p))
        {
            if (!virgin && !my_path.empty() && !is_dir(my_path))
            {
                if (mkdir(my_path.c_str() MKDIR_MODE) != 0)
                {
                    log << LOG_DATE
                        << "Warning: Failed to create directory \""
                        << my_path
                        << "\" due to error "
                        << strerror(errno)
                        << ENDL;

                    return false;
                }
            
                set_ownership(my_path, unix_mode, user, group);
            }
            my_path += SYSSLASH;
            p++;
            virgin = false;
        }
        else
        {
            my_path += *p++;
        }
    }

    return true;
}

std::string parse_path(const std::string& path)
{
    typedef char path_char_t;
    
    std::vector<path_char_t> buffer(path.size() + 1);
    const path_char_t *p = path.c_str();
    path_char_t *pp = &buffer[0];
    int cnt = 0;
    
    while (*p)
    {
        if (IS_SLASH(*p))
        {
            // /C/.. /C/./any  /C/any/. /C/any/../.any /C/any\..
            if (p[1] == '.')
            {
                if (!p[2] || IS_SLASH(p[2]))
                {
                    // /C/any/. style. Just ignore
                    p += 2;
                    continue;
                }
                
                if (p[2] == '.')
                {
                    if (!p[3] || IS_SLASH(p[3]))
                    {
                        // /C/any/.. /C/any/../test style. Move back one
                        if (cnt == 0)
                            throw exception();
                        
                        while (--cnt && !IS_SLASH(pp[cnt]))
                        {
                            assert(cnt >= 0);
                        }
                        p += 3;
                        continue;
                    }
                    else
                    {
                        const path_char_t *ppp = p;
                        while(*ppp == '.')
                            ++ppp;
                        
                        // Deny /......[/]
                        if (!*ppp 
                            || IS_SLASH(*ppp) 
                            // We deny some common characters with special meanings as well!
                            // /..*../
                            || (*ppp == '*') 
                            || (*ppp == '?')
                            || (*ppp == '~'))
                        {
                            // Possible exploit
                            throw exception();
                        }
                    }
                }
            }
        }
        // Default
        pp[cnt++] = *p++;
    }
    
    pp[cnt++] = 0;
    
    return (const char *)&buffer[0];
}

#ifdef WIN32
int get_registry_value(HKEY root, 
                       const char *key, 
                       const char *value, 
                       char *return_buf, 
                       DWORD *return_buf_len) {
    size_t tmp_len;
    DWORD error_status;
    HKEY registry_handle;
    char *tmp_buffer;
    DWORD key_type = 0;
    if ((error_status = RegOpenKeyEx(
        root, key, 0, KEY_QUERY_VALUE, &registry_handle)) == ERROR_SUCCESS) {
        if ((error_status = RegQueryValueEx(registry_handle, value, NULL, 
            &key_type, (LPBYTE)return_buf, return_buf_len)) == ERROR_SUCCESS) {
            if (key_type == REG_EXPAND_SZ) {
                tmp_len = strlen(return_buf) + 256;
                tmp_buffer = (char *)alloca(tmp_len);
                error_status = ExpandEnvironmentStrings(return_buf, tmp_buffer, 
                    tmp_len);
                strcpy(return_buf, tmp_buffer);
            }
        }
        RegCloseKey(registry_handle);
    }
    
    if (error_status == ERROR_SUCCESS)
        return 0;
    
    return -1;
}

void get_registry_value(HKEY root, 
                        const char *key, 
                        const char *value, 
                        std::string& return_value,
                        const char *def_value)
{
    char buf[1024 * 4];
    DWORD buflen = sizeof(buf);

    if (get_registry_value(root, key, value, buf, &buflen) == 0)
        return_value = buf;
    else
        return_value = def_value ? def_value : "";
}


void UnixTimeToFileTime(const time_t t, FILETIME& ft)
{
    LONGLONG ll;
    
    ll = Int32x32To64(t, 10000000) + 116444736000000000;
    ft.dwLowDateTime = (DWORD)ll;
    ft.dwHighDateTime = ll >> 32;
}

void FileTimeToUnixTime(const FILETIME& ft, time_t& t)
{
    LONGLONG ll = ((LONGLONG)ft.dwHighDateTime) << 32;
    ll = ll + ft.dwLowDateTime - 116444736000000000;
    t = (unsigned long) (ll/10000000L);
}

void FileTimeToSsftTime(const FILETIME& ft, ssft_date_t& t)
{
    LONGLONG ll = ((LONGLONG)ft.dwHighDateTime) << 32;
    ll = ll + ft.dwLowDateTime - 116444736000000000;
    t = (unsigned long) (ll/10000000L);
    t = t * 1000; 
}

#endif // WIN32

bool is_device(const char *name)
{
#ifdef WIN32
    WIN32_FIND_DATA info;
    static const FILETIME ft_zero = {0,0};
    HANDLE h;

    // Devices usually have date set to zero
    h = FindFirstFile(name, &info);
    if (h != INVALID_HANDLE_VALUE)
    {
        FindClose(h);

        if ((memcmp(&info.ftCreationTime, &ft_zero, sizeof(ft_zero)) == 0)
            && (info.nFileSizeHigh == 0)
            && (info.nFileSizeLow == 0))
            return true;
    }

#else
    
    ssft_stat_t st;
    if (ssft_stat(name, &st) == 0)
    {
      if (!S_ISREG(st.st_mode) && !S_ISDIR(st.st_mode))
	    return true;
    }
#endif

    // Don't look like a device
    return false;
}
